@@ -1,14 +1,17 @@
 """
-Open Generation and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - January 03, 2022
+Open Generation and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - January 19, 2022
 """
 
 import time
 import os
 import pandas as pd
 import altair as alt
-from   collections   import defaultdict
+import plotly.io as pio
+import plotly.graph_objs as go
 import matplotlib.pyplot as plt
+from   collections   import defaultdict
 from   pyomo.environ import Set
+from   colour        import Color
 
 
 # Definition of Pie plots
@@ -17,14 +20,14 @@ def PiePlots(df, Category, Value):
     OutputToPlot          = OutputToPlot.reset_index().rename(columns={'level_3': Category, '': Value})
     OutputToPlot[Value]   = round(OutputToPlot[Value], 1)
     OutputToPlot          = OutputToPlot[(OutputToPlot[[Value]] != 0).all(axis=1)]
-    OutputToPlot['Label'] = [OutputToPlot[Category][i] + ': ' + str(OutputToPlot[Value][i]) for i in OutputToPlot.index]
-    ComposedCategory      = Category + ':N'
-    ComposedValue         = Value    + ':Q'
+    OutputToPlot['Label'] = [OutputToPlot[Category][i]+': '+str(OutputToPlot[Value][i]) for i in OutputToPlot.index]
+    ComposedCategory      = Category+':N'
+    ComposedValue         = Value   +':Q'
 
     base  = alt.Chart(OutputToPlot).encode(theta=alt.Theta(ComposedValue, stack=True), color=alt.Color(ComposedCategory, legend=alt.Legend(title=Category))).properties(width=800, height=800)
     pie   = base.mark_arc(outerRadius=240)
     text  = base.mark_text(radius=300, size=15).encode(text='Label:N')
-    chart = pie + text
+    chart = pie+text
 
     return chart
 
@@ -35,16 +38,16 @@ def AreaPlots(scenario, period, df, Category, X, Y, OperationType):
     Results = Results.reset_index().rename(columns={'level_0': X, 'level_1': Category, 0: Y})
     # # Change the format of the LoadLevel
     Results[X] = Results[X].str[:19]
-    Results[X] = (Results[X] + '+01:00')
+    Results[X] = (Results[X]+'+01:00')
     Results[X] = pd.to_datetime(Results[X])
     Results[X] = Results[X].dt.strftime('%m-%d %H:%M+01:00')
     # Composed Names
-    C_C = Category + ':N'
-    C_X = 'monthdate('   + X + '):T'
+    C_C = Category+':N'
+    C_X = 'monthdate('  +X+'):T'
     if OperationType == 'average':
-        C_Y = 'average(' + Y + '):Q'
+        C_Y = 'average('+Y+'):Q'
     else:
-        C_Y = 'sum('     + Y + '):Q'
+        C_Y = 'sum('    +Y+'):Q'
 
     # Define and build the chart
     alt.data_transformers.disable_max_rows()
@@ -59,21 +62,21 @@ def AreaPlots(scenario, period, df, Category, X, Y, OperationType):
 
 
 # Definition of Circle plots
-def CirclePlots(scenario, period, df, Category, X, Y, OperationType):
+def LinePlots(scenario, period, df, Category, X, Y, OperationType):
     Results = df.loc[scenario,period,:,:]
     Results = Results.reset_index().rename(columns={'level_0': X, 'level_1': Category, 0: Y})
     # # Change the format of the LoadLevel
     Results[X] = Results[X].str[:19]
-    Results[X] = (Results[X] + '+01:00')
+    Results[X] = (Results[X]+'+01:00')
     Results[X] = pd.to_datetime(Results[X])
     Results[X] = Results[X].dt.strftime('%m-%d %H:%M+01:00')
     # Composed Names
-    C_C = Category + ':N'
-    C_X = 'monthdate('   + X + '):T'
+    C_C = Category+':N'
+    C_X = 'monthdate('  +X+'):T'
     if OperationType == 'average':
-        C_Y = 'average(' + Y + '):Q'
+        C_Y = 'average('+Y+'):Q'
     else:
-        C_Y = 'sum('     + Y + '):Q'
+        C_Y = 'sum('    +Y+'):Q'
 
     # Define and build the chart
     alt.data_transformers.disable_max_rows()
@@ -125,7 +128,7 @@ def InvestmentResults(DirName, CaseName, OptModel, mTEPES):
         OutputToFile.index = pd.MultiIndex.from_tuples(OutputToFile.index)
         OutputToFile.index.names = ['InitialNode','FinalNode','Circuit']
         OutputToFile = OutputToFile.reset_index()
-        OutputToFile.rename(columns={'vNetworkInvest': 'Investment Decision'}).to_csv(_path + '/oT_Result_NetworkInvestment_'+CaseName+'.csv', index=False, sep=',')
+        OutputToFile.rename(columns={'vNetworkInvest': 'Investment Decision'}).to_csv(_path+'/oT_Result_NetworkInvestment_'+CaseName+'.csv', index=False, sep=',')
 
         OutputResults_1 = pd.Series(data=[lt for ni,nf,cc,lt in mTEPES.lc*mTEPES.lt if (ni,nf,cc,lt) in mTEPES.pLineType], index=pd.Index(mTEPES.lc))
         OutputResults_1 = OutputResults_1.to_frame(name='LineType')
@@ -143,7 +146,7 @@ def InvestmentResults(DirName, CaseName, OptModel, mTEPES):
 
         OutputToFile = pd.Series(data=[(OptModel.vNetworkInvest[ni,nf,cc]()*mTEPES.pLineNTCFrw[ni,nf,cc]*1e3)/mTEPES.pLineLength[ni,nf,cc]() for ni,nf,cc in mTEPES.lc], index= pd.MultiIndex.from_tuples(list(mTEPES.lc)))
         OutputToFile = OutputToFile.to_frame(name='MW-km')
-        OutputToFile.to_csv(_path + '/oT_Result_NetworkInvestment_MWkm_'+CaseName+'.csv', sep=',')
+        OutputToFile.to_csv(_path+'/oT_Result_NetworkInvestment_MWkm_'+CaseName+'.csv', sep=',')
 
         OutputResults_1 = pd.Series(data=[lt for ni,nf,cc,lt in mTEPES.lc*mTEPES.lt if (ni,nf,cc,lt) in mTEPES.pLineType], index=pd.Index(mTEPES.lc))
         OutputResults_1 = OutputResults_1.to_frame(name='LineType')
@@ -237,7 +240,7 @@ def GenerationOperationResults(DirName, CaseName, OptModel, mTEPES):
         TechCurt.rename({'index': 'Technologies'}, axis=1, inplace=True)
 
         chart = alt.Chart(TechCurt).mark_bar().encode(x='Technologies', y='GWh').properties(width=600, height=400)
-        chart.save(_path + '/oT_Plot_TechnologyCurtailment_'+CaseName+'.html', embed_options={'renderer':'svg'})
+        chart.save(_path+'/oT_Plot_TechnologyCurtailment_'+CaseName+'.html', embed_options={'renderer':'svg'})
 
     OutputToFile = pd.Series(data=[OptModel.vTotalOutput[sc,p,n,g ]()*mTEPES.pDuration[n]*mTEPES.pLoadLevelWeight[n]() for sc,p,n,g  in mTEPES.sc*mTEPES.p*mTEPES.n*mTEPES.g ], index=pd.MultiIndex.from_tuples(mTEPES.sc*mTEPES.p*mTEPES.n*mTEPES.g ))
     OutputToFile.to_frame(name='GWh').reset_index().pivot_table(index=['level_0','level_1','level_2'],     columns='level_3', values='GWh'  ).rename_axis(['Scenario','Period','LoadLevel'], axis=0).rename_axis([None], axis=1).to_csv(_path+'/oT_Result_GenerationEnergy_'  +CaseName+'.csv', sep=',')
@@ -256,11 +259,17 @@ def GenerationOperationResults(DirName, CaseName, OptModel, mTEPES):
         TechCO2.rename({'index': 'Technologies'}, axis=1, inplace=True)
 
         chart = alt.Chart(TechCO2).mark_bar().encode(x='Technologies', y='MtCO2').properties(width=600, height=400)
-        chart.save(_path + '/oT_Plot_Emissions_'+CaseName+'.html', embed_options={'renderer': 'svg'})
+        chart.save(_path+'/oT_Plot_Emissions_'+CaseName+'.html', embed_options={'renderer': 'svg'})
 
     OutputToFile = pd.Series(data=[sum(OptModel.vTotalOutput[sc,p,n,g]() for g in mTEPES.g if (gt,g) in mTEPES.t2g)*1e3 for sc,p,n,gt in mTEPES.sc*mTEPES.p*mTEPES.n*mTEPES.gt], index=pd.MultiIndex.from_tuples(mTEPES.sc*mTEPES.p*mTEPES.n*mTEPES.gt))
     OutputToFile.to_frame(name='MW' ).reset_index().pivot_table(index=['level_0','level_1','level_2'], columns='level_3', values='MW').rename_axis(['Scenario','Period','LoadLevel'], axis=0).rename_axis([None], axis=1).to_csv(_path+'/oT_Result_TechnologyOutput_'+CaseName+'.csv', sep=',')
-
+    
+    TechnologyOutput = OutputToFile.loc[:,:,:,:]
+    
+    for sc,p in mTEPES.sc*mTEPES.p:
+        chart = AreaPlots(sc, p, TechnologyOutput, 'Technology', 'LoadLevel', 'MW', 'sum')
+        chart.save(_path+'/oT_Plot_TechnologyOutput_'+sc+'_'+p+'_'+CaseName+'.html', embed_options={'renderer': 'svg'})
+    
     OutputToFile = pd.Series(data=[OptModel.vTotalOutput[sc,p,n,g]()*mTEPES.pDuration[n]*mTEPES.pLoadLevelWeight[n]() for sc,p,n,g in mTEPES.sc*mTEPES.p*mTEPES.n*mTEPES.g ], index=pd.MultiIndex.from_tuples(mTEPES.sc*mTEPES.p*mTEPES.n*mTEPES.g ))
     OutputToFile = pd.Series(data=[sum(OutputToFile[sc,p,n,g] for g in mTEPES.g if (gt,g) in mTEPES.t2g) for sc,p,n,gt in mTEPES.sc*mTEPES.p*mTEPES.n*mTEPES.gt], index=pd.MultiIndex.from_tuples(mTEPES.sc*mTEPES.p*mTEPES.n*mTEPES.gt))
     OutputToFile.to_frame(name='GWh').reset_index().pivot_table(index=['level_0','level_1','level_2'], columns='level_3', values='GWh').rename_axis(['Scenario','Period','LoadLevel'], axis=0).rename_axis([None], axis=1).to_csv(_path+'/oT_Result_TechnologyEnergy_'+CaseName+'.csv', sep=',')
@@ -270,7 +279,7 @@ def GenerationOperationResults(DirName, CaseName, OptModel, mTEPES):
     TechnologyEnergy.reset_index(level=3, inplace=True)
 
     chart = PiePlots(TechnologyEnergy, 'Technology', '%')
-    chart.save(_path + '/oT_Plot_TechnologyEnergy_'+CaseName+'.html', embed_options={'renderer': 'svg'})
+    chart.save(_path+'/oT_Plot_TechnologyEnergy_'+CaseName+'.html', embed_options={'renderer': 'svg'})
 
     for ar in mTEPES.ar:
         if len(mTEPES.ar) > 1:
@@ -364,7 +373,7 @@ def ESSOperationResults(DirName, CaseName, OptModel, mTEPES):
 
         for sc,p in mTEPES.sc*mTEPES.p:
             chart = AreaPlots(sc, p, TechnologyCharge, 'Technology', 'LoadLevel', 'MW', 'sum')
-            chart.save(_path + '/oT_Plot_TechnologyCharge_'+sc+'_'+p+'_'+CaseName+'.html', embed_options={'renderer': 'svg'})
+            chart.save(_path+'/oT_Plot_TechnologyCharge_'+sc+'_'+p+'_'+CaseName+'.html', embed_options={'renderer': 'svg'})
 
     WritingResultsTime = time.time() - StartTime
     StartTime = time.time()
@@ -489,8 +498,8 @@ def MarginalResults(DirName, CaseName, OptModel, mTEPES):
     LSRMC = OutputToFile.loc[:,:]
 
     for sc,p in mTEPES.sc*mTEPES.p:
-        chart = CirclePlots(sc, p, LSRMC, 'Node', 'LoadLevel', 'EUR/MWh', 'average')
-        chart.save(_path + '/oT_Plot_LSRMC_'+sc+'_'+p+'_'+CaseName+'.html', embed_options={'renderer': 'svg'})
+        chart = LinePlots(sc, p, LSRMC, 'Node', 'LoadLevel', 'EUR/MWh', 'average')
+        chart.save(_path+'/oT_Plot_LSRMC_'+sc+'_'+p+'_'+CaseName+'.html', embed_options={'renderer': 'svg'})
 
     if sum(mTEPES.pReserveMargin[ar] for ar in mTEPES.ar):
         OutputToFile = pd.Series(data=[OptModel.dual[getattr(OptModel, 'eAdequacyReserveMargin_'+st)[ar]] for st,ar in mTEPES.st*mTEPES.ar if mTEPES.pReserveMargin[ar] and sum(1 for nr in mTEPES.nr if (ar,nr) in mTEPES.a2g) + sum(1 for es in mTEPES.es if (ar,es) in mTEPES.a2g)], index=pd.MultiIndex.from_tuples(list(mTEPES.st*list(getattr(OptModel, 'eAdequacyReserveMargin_'+st)))))
@@ -512,8 +521,8 @@ def MarginalResults(DirName, CaseName, OptModel, mTEPES):
         MarginalUpOperatingReserve = OutputToFile.loc[:,:]
 
         for sc,p in mTEPES.sc*mTEPES.p:
-            chart = CirclePlots(sc, p, MarginalUpOperatingReserve, 'Area', 'LoadLevel', 'EUR/MW', 'sum')
-            chart.save(_path + '/oT_Plot_MarginalOperatingReserveUpward_'+sc+'_'+p+'_'+CaseName+'.html', embed_options={'renderer': 'svg'})
+            chart = LinePlots(sc, p, MarginalUpOperatingReserve, 'Area', 'LoadLevel', 'EUR/MW', 'sum')
+            chart.save(_path+'/oT_Plot_MarginalOperatingReserveUpward_'+sc+'_'+p+'_'+CaseName+'.html', embed_options={'renderer': 'svg'})
 
     #%% outputting the down operating reserve marginal
     if sum(mTEPES.pOperReserveDw[sc,p,n,ar] for sc,p,n,ar in mTEPES.sc*mTEPES.p*mTEPES.n*mTEPES.ar) > 0 and (sum(1 for ar,nr in mTEPES.ar*mTEPES.nr if (ar,nr) in mTEPES.a2g and mTEPES.pIndOperReserve[nr] == 0) + sum(1 for ar,es in mTEPES.ar*mTEPES.es if (ar,es) in mTEPES.a2g and mTEPES.pIndOperReserve[es] == 0)) > 0:
@@ -532,8 +541,8 @@ def MarginalResults(DirName, CaseName, OptModel, mTEPES):
         MarginalDwOperatingReserve = OutputToFile.loc[:,:]
 
         for sc,p in mTEPES.sc*mTEPES.p:
-            chart = CirclePlots(sc, p, MarginalDwOperatingReserve, 'Area', 'LoadLevel', 'EUR/MW', 'sum')
-            chart.save(_path + '/oT_Plot_MarginalOperatingReserveDownward_'+sc+'_'+p+'_'+CaseName+'.html', embed_options={'renderer': 'svg'})
+            chart = LinePlots(sc, p, MarginalDwOperatingReserve, 'Area', 'LoadLevel', 'EUR/MW', 'sum')
+            chart.save(_path+'/oT_Plot_MarginalOperatingReserveDownward_'+sc+'_'+p+'_'+CaseName+'.html', embed_options={'renderer': 'svg'})
 
     #%% outputting the water values
     if len(mTEPES.es):
@@ -552,15 +561,15 @@ def MarginalResults(DirName, CaseName, OptModel, mTEPES):
         WaterValue = OutputToFile.loc[:,:]
 
         for sc,p in mTEPES.sc*mTEPES.p:
-            chart = CirclePlots(sc, p, WaterValue, 'Unit', 'LoadLevel', 'EUR/MWh', 'average')
-            chart.save(_path + '/oT_Plot_WaterValue_'+sc+'_'+p+'_'+CaseName+'.html', embed_options={'renderer': 'svg'})
+            chart = LinePlots(sc, p, WaterValue, 'Unit', 'LoadLevel', 'EUR/MWh', 'average')
+            chart.save(_path+'/oT_Plot_WaterValue_'+sc+'_'+p+'_'+CaseName+'.html', embed_options={'renderer': 'svg'})
 
     #%% Reduced cost for NetworkInvestment
     ReducedCostActivation = mTEPES.pIndBinGenInvest()*len(mTEPES.gc) + mTEPES.pIndBinNetInvest()*len(mTEPES.lc) + mTEPES.pIndBinGenOperat()*len(mTEPES.nr) + mTEPES.pIndBinLineCommit()*len(mTEPES.la)
     if len(mTEPES.lc) and not ReducedCostActivation:
         OutputToFile = pd.Series(data=[OptModel.rc[OptModel.vNetworkInvest[ni,nf,cc]] for ni,nf,cc in mTEPES.lc], index=pd.MultiIndex.from_tuples(getattr(OptModel, 'vNetworkInvest')))
         OutputToFile.index.names = ['InitialNode', 'FinalNode', 'Circuit']
-        OutputToFile.to_frame(name='p.u.').to_csv(_path + '/oT_Result_NetworkInvestment_ReducedCost_'+CaseName+'.csv', sep=',')
+        OutputToFile.to_frame(name='p.u.').to_csv(_path+'/oT_Result_NetworkInvestment_ReducedCost_'+CaseName+'.csv', sep=',')
 
     # %% Reduced cost for NetworkCommitment
     mTEPES.las = Set(initialize=mTEPES.ni*mTEPES.nf*mTEPES.cc, ordered=False, doc='all real lines with switching decision', filter=lambda mTEPES,ni,nf,cc: (ni,nf,cc) in mTEPES.pLineX and mTEPES.pIndBinLineSwitch[ni,nf,cc])
@@ -664,9 +673,11 @@ def EconomicResults(DirName, CaseName, OptModel, mTEPES):
                 OutputToGenRev       = pd.Series(data=[OptModel.dual[getattr(OptModel, 'eBalance_'+st)[sc,p,n,nd]]/mTEPES.pScenProb[sc] * OptModel.vTotalOutput   [sc,p,n,gc]() for sc,p,n,nd,gc in mTEPES.sc*mTEPES.p*mTEPES.n*mTEPES.n2g if gc in mTEPES.gc], index=pd.MultiIndex.from_tuples([(sc,p,n,nd,gc) for sc,p,n,nd,gc in mTEPES.sc*mTEPES.p*mTEPES.n*mTEPES.n2g if gc in mTEPES.gc]))
                 OutputChargeRevESS   = pd.Series(data=[OptModel.dual[getattr(OptModel, 'eBalance_'+st)[sc,p,n,nd]]/mTEPES.pScenProb[sc] * OptModel.vESSTotalCharge[sc,p,n,gc]() for sc,p,n,nd,gc in mTEPES.sc*mTEPES.p*mTEPES.n*mTEPES.n2g if gc in mTEPES.gc for ot in mTEPES.ot if (ot,gc) in mTEPES.t2g], index=pd.MultiIndex.from_tuples([(sc,p,n,nd,gc) for sc,p,n,nd,gc in mTEPES.sc*mTEPES.p*mTEPES.n*mTEPES.n2g if gc in mTEPES.gc for ot in mTEPES.ot if (ot,gc) in mTEPES.t2g]))
                 OutputChargeRevRES   = pd.Series(data=[OptModel.dual[getattr(OptModel, 'eBalance_'+st)[sc,p,n,nd]]/mTEPES.pScenProb[sc] * 0 for sc,p,n,nd,gc in mTEPES.sc*mTEPES.p*mTEPES.n*mTEPES.n2g if gc in mTEPES.gc for rt in mTEPES.rt if (rt,gc) in mTEPES.t2g], index=pd.MultiIndex.from_tuples([(sc,p,n,nd,gc) for sc,p,n,nd,gc in mTEPES.sc*mTEPES.p*mTEPES.n*mTEPES.n2g if gc in mTEPES.gc for rt in mTEPES.rt if (rt,gc) in mTEPES.t2g]))
+                OutputChargeRevThr   = pd.Series(data=[OptModel.dual[getattr(OptModel, 'eBalance_'+st)[sc,p,n,nd]]/mTEPES.pScenProb[sc] * 0 for sc,p,n,nd,gc in mTEPES.sc*mTEPES.p*mTEPES.n*mTEPES.n2g if gc in mTEPES.gc for ot in mTEPES.ot if (ot,gc) not in mTEPES.t2g], index=pd.MultiIndex.from_tuples([(sc,p,n,nd,gc) for sc,p,n,nd,gc in mTEPES.sc*mTEPES.p*mTEPES.n*mTEPES.n2g if gc in mTEPES.gc for ot in mTEPES.ot if (ot,gc) not in mTEPES.t2g]))
             GenRev.append   (OutputToGenRev    )
             ChargeRev.append(OutputChargeRevESS)
             ChargeRev.append(OutputChargeRevRES)
+            ChargeRev.append(OutputChargeRevThr)
         mTEPES.del_component(mTEPES.n)
         mTEPES.n = Set(initialize=mTEPES.nn, ordered=True, doc='load levels', filter=lambda mTEPES,nn: nn in mTEPES.pDuration)
         GenRev    = pd.concat(GenRev)
@@ -773,97 +784,158 @@ def EconomicResults(DirName, CaseName, OptModel, mTEPES):
     InvCost = pd.Series(data=[mTEPES.pGenInvestCost[gc] for gc in mTEPES.gc], index= pd.Index(mTEPES.gc))
     Balance = pd.Series(data=[GenRev[gc]+ChargeRev[gc]+UpRev[gc]+DwRev[gc]+ResRev[gc]-InvCost[gc] for gc in mTEPES.gc], index= pd.Index(mTEPES.gc))
     CostRecoveryESS = pd.concat([GenRev,ChargeRev,UpRev,DwRev,ResRev,InvCost,Balance], axis=1, keys=['Generation revenue [MEUR]','Consumption revenue [MEUR]','Up reserve revenue [MEUR]','Down reserve revenue [MEUR]','Reserve capacity revenue [MEUR]','Investment Cost [MEUR]','Balance [MEUR]'])
-    CostRecoveryESS.stack().to_frame(name='MEUR').reset_index().pivot_table(values='MEUR', index=['level_1'], columns=['level_0']).rename_axis([None], axis=0).rename_axis([None], axis=1).to_csv(_path + '/oT_Result_CostRecovery_'+CaseName+'.csv', index=True, sep=',')
+    CostRecoveryESS.stack().to_frame(name='MEUR').reset_index().pivot_table(values='MEUR', index=['level_1'], columns=['level_0']).rename_axis([None], axis=0).rename_axis([None], axis=1).to_csv(_path+'/oT_Result_CostRecovery_'+CaseName+'.csv', index=True, sep=',')
 
     WritingResultsTime = time.time() - StartTime
     print('Writing             economic results   ... ', round(WritingResultsTime), 's')
 
 
 def NetworkMapResults(DirName, CaseName, OptModel, mTEPES):
-    #%% plotting the network in a map
+    # %% plotting the network in a map
     _path = os.path.join(DirName, CaseName)
+    DIR   = os.path.dirname(__file__)
     StartTime = time.time()
 
-    import cartopy.crs          as ccrs
-    import cartopy.io.img_tiles as cimgt
-    import cartopy.feature      as cfeature
-    from matplotlib.transforms import offset_copy
+    # Sub functions
+    mTEPES.del_component(mTEPES.sc)
+    mTEPES.del_component(mTEPES.p )
+    mTEPES.del_component(mTEPES.st)
+    mTEPES.del_component(mTEPES.n )
+    mTEPES.sc = Set(initialize=mTEPES.scc, ordered=True, doc='scenarios',   filter=lambda mTEPES,scc: scc in mTEPES.scc and mTEPES.pScenProb   [scc])
+    mTEPES.p  = Set(initialize=mTEPES.pp,  ordered=True, doc='periods',     filter=lambda mTEPES,pp : pp                                            )
+    mTEPES.st = Set(initialize=mTEPES.stt, ordered=True, doc='stages',      filter=lambda mTEPES,stt: stt in mTEPES.stt and mTEPES.pStageWeight[stt] and sum(1 for (stt, nn) in mTEPES.s2n))
+    mTEPES.n  = Set(initialize=mTEPES.nn,  ordered=True, doc='load levels', filter=lambda mTEPES,nn : nn  in                mTEPES.pDuration        )
 
-    # Create a Stamen terrain background instance.
-    stamen_terrain = cimgt.Stamen('terrain-background')
+    def oT_make_series(_var, _sets, _factor):
+        return pd.Series(data=[_var[sc, p, n, ni, nf, cc]()*_factor for sc, p, n, ni, nf, cc in _sets],
+                         index=pd.MultiIndex.from_tuples(list(_sets)))
 
-    fig = plt.figure()
 
-    # Create a GeoAxes in the tile's projection.
-    ax = fig.add_subplot(1, 1, 1, projection=stamen_terrain.crs)
+    def oT_selecting_data(sc, p, n):
+        # Nodes data
+        pio.renderers.default = 'chrome'
 
-    # Limit the extent of the map to a small longitude/latitude range.
-    ax.set_extent((min(mTEPES.pNodeLon.values()) - 2, max(mTEPES.pNodeLon.values()) + 2,
-                   min(mTEPES.pNodeLat.values()) - 2, max(mTEPES.pNodeLat.values()) + 2), crs=ccrs.Geodetic())
+        loc_df = pd.Series(data=[mTEPES.pNodeLat[i] for i in mTEPES.nd], index=mTEPES.nd).to_frame(name='Lat')
+        loc_df['Lon'   ] =  0.0
+        loc_df['Zone'  ] =  0.0
+        loc_df['Demand'] =  0.0
+        loc_df['Size'  ] = 15.0
 
-    ax.add_feature(cfeature.BORDERS, linewidth=1  )
-    ax.add_feature(cfeature.STATES,  linewidth=0.1)
-    # Add the Stamen data at zoom level 8.
-    ax.add_image(stamen_terrain, 8)
+        for nd,zn in mTEPES.ndzn:
+            loc_df['Lon'   ][nd] = mTEPES.pNodeLon[nd]
+            loc_df['Zone'  ][nd] = zn
+            loc_df['Demand'][nd] = mTEPES.pDemand[sc,p,n,nd]*1e3
 
-    # Use the cartopy interface to create a matplotlib transform object
-    # for the Geodetic coordinate system. We will use this along with
-    # matplotlib's offset_copy function to define a coordinate system which
-    # translates the text by 25 pixels to the left.
-    geodetic_transform = ccrs.Geodetic()._as_mpl_transform(ax)
-    text_transform = offset_copy(geodetic_transform, units='dots', x=-10)
+        loc_df = loc_df.reset_index().rename(columns={'Type': 'Scenario'}, inplace=False)
 
-    # node name
-    for nd in mTEPES.nd:
-        # plt.annotate(nd, [mTEPES.pNodeLon[nd], mTEPES.pNodeLat[nd]])
-        # Add text 25 pixels to the left of each node.
-        ax.text(mTEPES.pNodeLon[nd], mTEPES.pNodeLat[nd], nd, fontsize=8, verticalalignment='center', horizontalalignment='right', transform=text_transform, bbox=dict(facecolor='sandybrown', alpha=0.5, boxstyle='round'))
+        # Edges data
+        OutputToFile = oT_make_series(OptModel.vFlow, mTEPES.sc * mTEPES.p * mTEPES.n * mTEPES.la, 1e3)
+        OutputToFile.index.names = ['Scenario', 'Period', 'LoadLevel', 'InitialNode', 'FinalNode', 'Circuit']
+        OutputToFile = OutputToFile.to_frame(name='MW')
 
-    #%% colors of the lines according to the ENTSO-E color code
-    # existing lines
-    for ni,nf,cc in mTEPES.le:
-        if (ni,nf,cc) in mTEPES.lea:
-            if mTEPES.pLineVoltage[ni,nf,cc] > 700 and mTEPES.pLineVoltage[ni,nf,cc] <= 900:
-                ax.plot([mTEPES.pNodeLon[ni], mTEPES.pNodeLon[nf]], [mTEPES.pNodeLat[ni], mTEPES.pNodeLat[nf]], color='blue'   , linewidth=2  , marker='o', linestyle='solid', transform=ccrs.PlateCarree())
-            if mTEPES.pLineVoltage[ni,nf,cc] > 500 and mTEPES.pLineVoltage[ni,nf,cc] <= 700:
-                ax.plot([mTEPES.pNodeLon[ni], mTEPES.pNodeLon[nf]], [mTEPES.pNodeLat[ni], mTEPES.pNodeLat[nf]], color='#ff8000', linewidth=2  , marker='o', linestyle='solid', transform=ccrs.PlateCarree())
-            if mTEPES.pLineVoltage[ni,nf,cc] > 350 and mTEPES.pLineVoltage[ni,nf,cc] <= 500:
-                ax.plot([mTEPES.pNodeLon[ni], mTEPES.pNodeLon[nf]], [mTEPES.pNodeLat[ni], mTEPES.pNodeLat[nf]], color='red'    , linewidth=1  , marker='o', linestyle='solid', transform=ccrs.PlateCarree())
-            if mTEPES.pLineVoltage[ni,nf,cc] > 290 and mTEPES.pLineVoltage[ni,nf,cc] <= 350:
-                ax.plot([mTEPES.pNodeLon[ni], mTEPES.pNodeLon[nf]], [mTEPES.pNodeLat[ni], mTEPES.pNodeLat[nf]], color='green'  , linewidth=0.4, marker='o', linestyle='solid', transform=ccrs.PlateCarree())
-            if mTEPES.pLineVoltage[ni,nf,cc] > 200 and mTEPES.pLineVoltage[ni,nf,cc] <= 290:
-                ax.plot([mTEPES.pNodeLon[ni], mTEPES.pNodeLon[nf]], [mTEPES.pNodeLat[ni], mTEPES.pNodeLat[nf]], color='green'  , linewidth=0.4, marker='o', linestyle='solid', transform=ccrs.PlateCarree())
-            if                                         mTEPES.pLineVoltage[ni,nf,cc] <= 200:
-                ax.plot([mTEPES.pNodeLon[ni], mTEPES.pNodeLon[nf]], [mTEPES.pNodeLat[ni], mTEPES.pNodeLat[nf]], color='#ff6300', linewidth=0.4, marker='o', linestyle='solid', transform=ccrs.PlateCarree())
-        else:
-            ax.plot    ([mTEPES.pNodeLon[ni], mTEPES.pNodeLon[nf]], [mTEPES.pNodeLat[ni], mTEPES.pNodeLat[nf]], color='magenta', linewidth=1  , marker='o', linestyle='solid', transform=ccrs.PlateCarree())
-    # candidate lines
-    for ni,nf,cc in mTEPES.lc:
-        if (ni,nf,cc) in mTEPES.lca and OptModel.vNetworkInvest[ni,nf,cc]() > 0:
-            if mTEPES.pLineVoltage[ni,nf,cc] > 700 and mTEPES.pLineVoltage[ni,nf,cc] <= 900:
-                ax.plot([mTEPES.pNodeLon[ni], mTEPES.pNodeLon[nf]], [mTEPES.pNodeLat[ni], mTEPES.pNodeLat[nf]], color='blue'   , linewidth=2  , marker='o', linestyle='dashed', transform=ccrs.PlateCarree())
-            if mTEPES.pLineVoltage[ni,nf,cc] > 500 and mTEPES.pLineVoltage[ni,nf,cc] <= 700:
-                ax.plot([mTEPES.pNodeLon[ni], mTEPES.pNodeLon[nf]], [mTEPES.pNodeLat[ni], mTEPES.pNodeLat[nf]], color='#ff8000', linewidth=2  , marker='o', linestyle='dashed', transform=ccrs.PlateCarree())
-            if mTEPES.pLineVoltage[ni,nf,cc] > 350 and mTEPES.pLineVoltage[ni,nf,cc] <= 500:
-                ax.plot([mTEPES.pNodeLon[ni], mTEPES.pNodeLon[nf]], [mTEPES.pNodeLat[ni], mTEPES.pNodeLat[nf]], color='red'    , linewidth=1  , marker='o', linestyle='dashed', transform=ccrs.PlateCarree())
-            if mTEPES.pLineVoltage[ni,nf,cc] > 290 and mTEPES.pLineVoltage[ni,nf,cc] <= 350:
-                ax.plot([mTEPES.pNodeLon[ni], mTEPES.pNodeLon[nf]], [mTEPES.pNodeLat[ni], mTEPES.pNodeLat[nf]], color='green'  , linewidth=0.4, marker='o', linestyle='dashed', transform=ccrs.PlateCarree())
-            if mTEPES.pLineVoltage[ni,nf,cc] > 200 and mTEPES.pLineVoltage[ni,nf,cc] <= 290:
-                ax.plot([mTEPES.pNodeLon[ni], mTEPES.pNodeLon[nf]], [mTEPES.pNodeLat[ni], mTEPES.pNodeLat[nf]], color='green'  , linewidth=0.4, marker='o', linestyle='dashed', transform=ccrs.PlateCarree())
-            if                                         mTEPES.pLineVoltage[ni,nf,cc] <= 200:
-                ax.plot([mTEPES.pNodeLon[ni], mTEPES.pNodeLon[nf]], [mTEPES.pNodeLat[ni], mTEPES.pNodeLat[nf]], color='#ff6300', linewidth=0.4, marker='o', linestyle='dashed', transform=ccrs.PlateCarree())
-        else:
-            ax.plot    ([mTEPES.pNodeLon[ni], mTEPES.pNodeLon[nf]], [mTEPES.pNodeLat[ni], mTEPES.pNodeLat[nf]], color='magenta', linewidth=1  , marker='o', linestyle='dashed', transform=ccrs.PlateCarree())
+        line_df = pd.Series(data=[mTEPES.pLineNTCFrw[i]*1e3 for i in mTEPES.la], index=pd.MultiIndex.from_tuples(list(mTEPES.la))).to_frame(name='NTC')
+        line_df['vFlow'      ] = 0.0
+        line_df['utilization'] = 0.0
+        line_df['color'      ] = 0.0
+        line_df['voltage'    ] = 0.0
+        line_df['width'      ] = 0.0
+        line_df['lon'        ] = 0.0
+        line_df['lat'        ] = 0.0
+        line_df['ni'         ] = 0.0
+        line_df['nf'         ] = 0.0
+        line_df['cc'         ] = 0.0
 
-    # # line NTC
-    # for ni,nf,cc in mTEPES.la:
-    #     ax.annotate(round(mTEPES.pLineNTC[ni,nf,cc]*1e3), [(mTEPES.pNodeLon[ni]+mTEPES.pNodeLon[nf])/2, (mTEPES.pNodeLat[ni]+mTEPES.pNodeLat[nf])/2])
+        line_df = line_df.groupby(level=[0, 1]).sum()
+        ncolors = 11
+        colors = list(Color('lightgreen').range_to(Color('darkred'), ncolors))
+        colors = ['rgb'+str(x.rgb) for x in colors]
 
-    plt.title('Network Map')
-    # # Place a legend to the right of this smaller subplot.
-    # plt.legend(bbox_to_anchor=(1.05, 1), loc='upper left', borderaxespad=0.)
-    # plt.show()
-    plt.savefig(_path+'/oT_Plot_MapNetwork_'+CaseName+'.png', bbox_inches=None, dpi=1200)
+        for ni, nf, cc in mTEPES.la:
+            line_df['vFlow'      ][ni, nf] += OutputToFile['MW'][sc, p, n, ni, nf, cc]
+            line_df['utilization'][ni, nf] = abs(line_df['vFlow'][ni, nf]/line_df['NTC'][ni, nf])*100
+            line_df['lon'        ][ni, nf] = (mTEPES.pNodeLon[ni]+mTEPES.pNodeLon[nf]) * 0.5
+            line_df['lat'        ][ni, nf] = (mTEPES.pNodeLat[ni]+mTEPES.pNodeLat[nf]) * 0.5
+            line_df['ni'         ][ni, nf] = ni
+            line_df['nf'         ][ni, nf] = nf
+            line_df['cc'         ][ni, nf] += 1
+
+            if  0.0 <= line_df['utilization'][ni, nf] <= 10.0:
+                line_df['color'][ni, nf] = colors[0]
+            if 10.0 < line_df['utilization'][ni, nf] <= 20.0:
+                line_df['color'][ni, nf] = colors[1]
+            if 20.0 < line_df['utilization'][ni, nf] <= 30.0:
+                line_df['color'][ni, nf] = colors[2]
+            if 30.0 < line_df['utilization'][ni, nf] <= 40.0:
+                line_df['color'][ni, nf] = colors[3]
+            if 40.0 < line_df['utilization'][ni, nf] <= 50.0:
+                line_df['color'][ni, nf] = colors[4]
+            if 50.0 < line_df['utilization'][ni, nf] <= 60.0:
+                line_df['color'][ni, nf] = colors[5]
+            if 60.0 < line_df['utilization'][ni, nf] <= 70.0:
+                line_df['color'][ni, nf] = colors[6]
+            if 70.0 < line_df['utilization'][ni, nf] <= 80.0:
+                line_df['color'][ni, nf] = colors[7]
+            if 80.0 < line_df['utilization'][ni, nf] <= 90.0:
+                line_df['color'][ni, nf] = colors[8]
+            if 90.0 < line_df['utilization'][ni, nf] <= 100.0:
+                line_df['color'][ni, nf] = colors[9]
+            if 100.0 < line_df['utilization'][ni, nf]:
+                line_df['color'][ni, nf] = colors[10]
+
+            line_df['voltage'][ni, nf] = mTEPES.pLineVoltage[ni, nf, cc]
+            if   700 < line_df['voltage'][ni, nf] <= 900:
+                line_df['width'][ni, nf] = 4
+            elif 500 < line_df['voltage'][ni, nf] <= 700:
+                line_df['width'][ni, nf] = 3
+            elif 350 < line_df['voltage'][ni, nf] <= 500:
+                line_df['width'][ni, nf] = 2.5
+            elif 290 < line_df['voltage'][ni, nf] <= 350:
+                line_df['width'][ni, nf] = 2
+            elif 200 < line_df['voltage'][ni, nf] <= 290:
+                line_df['width'][ni, nf] = 1.5
+            elif  50 < line_df['voltage'][ni, nf] <= 200:
+                line_df['width'][ni, nf] = 1
+            else:
+                line_df['width'][ni, nf] = 0.5
+
+        # Rounding to decimals
+        line_df = line_df.round(decimals=2)
+
+        return loc_df, line_df
+
+    sc = list(mTEPES.sc)[0]
+    p  = list(mTEPES.p )[0]
+    n  = list(mTEPES.n )[0]
+
+    loc_df, line_df = oT_selecting_data(sc, p, n)
+
+    # Making the network
+    # Get node position dict
+    x, y = loc_df['Lon'].values, loc_df['Lat'].values
+    pos_dict = {}
+    for index, iata in enumerate(loc_df['index']):
+        pos_dict[iata] = (x[index], y[index])
+
+    # Setting up the figure
+    token = open(DIR+'/openTEPES.mapbox_token').read()
+
+    fig = go.Figure()
+
+    # Add nodes
+    fig.add_trace(go.Scattermapbox(lat=loc_df['Lat'], lon=loc_df['Lon'], mode='markers', marker=go.scattermapbox.Marker(size=loc_df['Size']*10, sizeref=1.1, sizemode='area', color='LightSkyBlue',), hoverinfo='text', text='<br>Zone: '+loc_df['index']+'<br>[Lon, Lat]: '+'('+loc_df['Lon'].astype(str)+', '+loc_df['Lat'].astype(str)+')'+'<br>Zone: '+loc_df['Zone']+'<br>Demand: '+loc_df['Demand'].astype(str)+' MW',))
+
+    # Add edges
+    for ni, nf, cc in mTEPES.la:
+        fig.add_trace(go.Scattermapbox(lon=[pos_dict[ni][0], pos_dict[nf][0]], lat=[pos_dict[ni][1], pos_dict[nf][1]], mode='lines+markers', marker=dict(size=0, showscale=True, colorbar={'title': 'Utilization [%]', 'titleside': 'top', 'thickness': 8, 'ticksuffix': '%'}, colorscale=[[0, 'lightgreen'], [1, 'darkred']], cmin=0, cmax=100,), line=dict(width=line_df['width'][ni, nf], color=line_df['color'][ni, nf]), opacity=1, hoverinfo='text', textposition='middle center',))
+
+    # Add legends related to the lines
+    fig.add_trace(go.Scattermapbox(lat=line_df['lat'], lon=line_df['lon'], mode='markers', marker=go.scattermapbox.Marker(size=20, sizeref=1.1, sizemode='area', color='LightSkyBlue',), opacity=0, hoverinfo='text', text='<br>Line: '+line_df['ni']+' --> '+line_df['nf']+'<br># Of circuits: '+line_df['cc'].astype(str)+'<br>Total NTC: '+line_df['NTC'].astype(str)+'<br>Power flow: '+line_df['vFlow'].astype(str)+'<br>Utilization [%]: '+line_df['utilization'].astype(str),))
+
+    # Setting up the layout
+    fig.update_layout(title={'text': 'Power Network: '+str(CaseName)+'<br>Scenario: '+str(sc)+'; Period: '+str(p)+'; LoadLevel: '+str(n), 'x': 0.5, 'xanchor': 'center', 'yanchor': 'top'}, font=dict(size=14), hovermode='closest', geo=dict(projection_type='azimuthal equal area', showland=True,), mapbox=dict(style='dark', accesstoken=token, bearing=0, center=dict(lat=(loc_df['Lat'].max()+loc_df['Lat'].min())*0.5, lon=(loc_df['Lon'].max()+loc_df['Lon'].min())*0.5), pitch=0, zoom=5), showlegend=False,)
+
+    # Saving the figure
+    fig.write_html(_path+'/oT_Plot_MapNetwork_'+CaseName+'.html')
 
     PlottingNetMapsTime = time.time() - StartTime
     print('Plotting network maps                  ... ', round(PlottingNetMapsTime), 's')
